import { Dialog } from '@jupyterlab/apputils';
import { JSONObject } from '@phosphor/coreutils';
import { Widget } from '@phosphor/widgets';
/**
 * Connection data for the mapd browser client.
 */
export interface IMapDConnectionData extends JSONObject {
    /**
     * The host of the connection, e.g. `metis.mapd.com`.
     */
    host: string;
    /**
     * The protocol to use, e.g. `https`.
     */
    protocol: string;
    /**
     * The port to use, e.g. `443`.
     */
    port: string;
    /**
     * The user name.
     */
    user: string;
    /**
     * The database name.
     */
    dbName: string;
    /**
     * The password for the connection.
     */
    password: string;
}
/**
 * Show a dialog for entering MapD connection data.
 */
export declare function showConnectionDialog(oldConnection?: IMapDConnectionData): Promise<IMapDConnectionData>;
/**
 * A dialog for entering MapD connection data.
 */
export declare class MapDConnectionDialog extends Widget implements Dialog.IBodyWidget<IMapDConnectionData> {
    constructor(oldData?: IMapDConnectionData);
    getValue(): IMapDConnectionData;
    private _user;
    private _password;
    private _database;
    private _host;
    private _protocol;
    private _port;
}
