import { DataModel } from '@phosphor/datagrid';
import { Widget } from '@phosphor/widgets';
import { IMapDConnectionData } from './connection';
import 'mapd-connector/dist/browser-connector.js';
/**
 * A widget that hosts a phosphor grid with a MapD dataset.
 */
export declare class MapDGrid extends Widget {
    /**
     * Construct a new MapDGrid widget.
     */
    constructor(connection?: IMapDConnectionData);
    /**
     * The current connection data for the viewer.
     */
    connection: IMapDConnectionData;
    /**
     * Update the underlying data model with a new query and connection.
     *
     * If the update fails, either due to a connection failure or a query
     * validation failure, it shows the error in the panel.
     */
    private _updateModel(connection, query);
    private _model;
    private _grid;
    private _gridStyle;
    private _toolbar;
    private _content;
    private _error;
}
/**
 * A data model for a query.
 */
export declare class MapDTableModel extends DataModel {
    /**
     * Construct a new data model.
     */
    constructor();
    /**
     * Get the number of rows for the model.
     */
    rowCount(region: DataModel.RowRegion): number;
    /**
     * Get the number of columns for the model.
     */
    columnCount(region: DataModel.ColumnRegion): number;
    /**
     * The current connection data for the model.
     */
    readonly connection: IMapDConnectionData | undefined;
    /**
     * The current query for the viewer.
     */
    readonly query: string;
    /**
     * Get data from the model.
     */
    data(region: DataModel.CellRegion, row: number, column: number): any;
    /**
     * Update the model with new connection data or a new query.
     *
     * @param connection - the connection data to use.
     *
     * @param query - the query to use.
     *
     * @returns a promsise that resolves when the model has been updated,
     *   and the connection and query data have been validated. It throws
     *   an error if the validation fails.
     */
    updateModel(connection: IMapDConnectionData, query: string): Promise<void>;
    /**
     * Reset the model. Should be called when either
     * the query or the connection data change.
     */
    private _updateModel();
    /**
     * Fetch a block with a given index into memory.
     */
    private _fetchBlock(index);
    /**
     * Free references to a block when it is no longer needed.
     */
    private _freeBlock(index);
    /**
     * If we are not chunking the data, then just load the whole thing,
     * limited by DEFAULT_LIMIT.
     */
    private _fetchDataset();
    private _query;
    private _connection;
    private _fieldNames;
    private _dataBlocks;
    private _dataset;
    private _pending;
    private _tableLength;
    private _maxBlock;
    private _streaming;
}
