"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var coreutils_1 = require("@phosphor/coreutils");
var datagrid_1 = require("@phosphor/datagrid");
var widgets_1 = require("@phosphor/widgets");
var apputils_1 = require("@jupyterlab/apputils");
var connection_1 = require("./connection");
require("mapd-connector/dist/browser-connector.js");
/**
 * The default block size whenn streaming blocks of the query results.
 */
var BLOCK_SIZE = 50000;
/**
 * The default limit for standalone requests.
 */
var DEFAULT_LIMIT = 50000;
/**
 * A widget that hosts a phosphor grid with a MapD dataset.
 */
var MapDGrid = /** @class */ (function (_super) {
    __extends(MapDGrid, _super);
    /**
     * Construct a new MapDGrid widget.
     */
    function MapDGrid(connection) {
        var _this = _super.call(this) || this;
        // Create the Layout
        _this.layout = new widgets_1.PanelLayout();
        _this._toolbar = new apputils_1.Toolbar();
        _this._toolbar.addClass('mapd-MapD-toolbar');
        _this._content = new widgets_1.StackedPanel();
        _this._content.addClass('mapd-MapDViewer-content');
        _this._error = new widgets_1.Widget({ node: document.createElement('pre') });
        _this._error.addClass('mapd-ErrorMessage');
        _this.layout.addWidget(_this._toolbar);
        _this.layout.addWidget(_this._content);
        _this.layout.addWidget(_this._error);
        // Create the data model
        _this._model = new MapDTableModel();
        // Create the grid
        var headerRenderer = new datagrid_1.TextRenderer({
            font: 'bold 14px sans-serif',
            horizontalAlignment: 'left'
        });
        var bodyRenderer = new datagrid_1.TextRenderer({
            horizontalAlignment: 'right'
        });
        _this._gridStyle = __assign({}, datagrid_1.DataGrid.defaultStyle, { rowBackgroundColor: function (i) { return i % 2 === 0 ? 'rgba(34, 167, 240, 0.2)' : ''; } });
        _this._grid = new datagrid_1.DataGrid({
            style: _this._gridStyle,
            baseRowSize: 24,
            baseColumnSize: 96,
            baseColumnHeaderSize: 24,
            baseRowHeaderSize: 64,
        });
        _this._grid.cellRenderers.set('body', {}, bodyRenderer);
        _this._grid.cellRenderers.set('column-header', {}, headerRenderer);
        _this._grid.model = _this._model;
        _this._content.addWidget(_this._grid);
        _this._content.hide(); // Initially hide the grid until we set the query.
        // Create the query input box
        var queryInput = document.createElement('input');
        queryInput.value = '';
        queryInput.placeholder = 'SQL Query';
        var queryInputWidget = new widgets_1.Widget({ node: queryInput });
        // Add an `Enter` keydown handler for the input field.
        queryInput.onkeydown = function (event) {
            switch (event.keyCode) {
                case 13:// Enter
                    event.stopPropagation();
                    event.preventDefault();
                    _this._updateModel(_this._model.connection, queryInput.value);
                    break;
                default:
                    break;
            }
        };
        // Create the toolbar.
        _this._toolbar.addItem('QueryInput', queryInputWidget);
        _this._toolbar.addItem('Query', new apputils_1.ToolbarButton({
            className: 'jp-RunIcon',
            onClick: function () {
                _this._updateModel(_this._model.connection, queryInput.value);
            },
            tooltip: 'Query'
        }));
        _this._toolbar.addItem('Connect', new apputils_1.ToolbarButton({
            className: 'mapd-MapD-logo',
            onClick: function () {
                connection_1.showConnectionDialog(_this._model.connection).then(function (connection) {
                    _this._updateModel(connection, _this._model.query);
                });
            },
            tooltip: 'Enter MapD Connection Data'
        }));
        // Initialize the data model.
        _this._updateModel(connection, '');
        return _this;
    }
    Object.defineProperty(MapDGrid.prototype, "connection", {
        /**
         * The current connection data for the viewer.
         */
        get: function () {
            return this._model.connection;
        },
        set: function (value) {
            this._updateModel(value, this._model.query);
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Update the underlying data model with a new query and connection.
     *
     * If the update fails, either due to a connection failure or a query
     * validation failure, it shows the error in the panel.
     */
    MapDGrid.prototype._updateModel = function (connection, query) {
        var _this = this;
        var hasQuery = query !== '';
        this._model.updateModel(connection, query).then(function () {
            _this._content.setHidden(!hasQuery);
            _this._error.node.textContent = '';
        }).catch(function (err) {
            _this._content.hide();
            _this._error.node.textContent = err ? (err.message || err) : 'Error';
        });
    };
    return MapDGrid;
}(widgets_1.Widget));
exports.MapDGrid = MapDGrid;
/**
 * A data model for a query.
 */
var MapDTableModel = /** @class */ (function (_super) {
    __extends(MapDTableModel, _super);
    /**
     * Construct a new data model.
     */
    function MapDTableModel() {
        var _this = _super.call(this) || this;
        _this._query = '';
        _this._dataBlocks = {};
        _this._dataset = null;
        _this._pending = new Set();
        _this._tableLength = -1;
        _this._maxBlock = Infinity;
        _this._streaming = false;
        _this._updateModel();
        return _this;
    }
    /**
     * Get the number of rows for the model.
     */
    MapDTableModel.prototype.rowCount = function (region) {
        if (region === 'column-header') {
            return 1;
        }
        // If we have found the length of the table, return that.
        if (this._tableLength > 0) {
            return this._tableLength;
        }
        // If we don't know the length, try to infer it from the currently
        // loaded blocks.
        var indices = Object.keys(this._dataBlocks).map(function (key) { return Number(key); });
        if (indices.length === 0) {
            return 0;
        }
        var maxIndex = Math.max.apply(Math, indices);
        return BLOCK_SIZE * maxIndex + this._dataBlocks[maxIndex].length;
    };
    /**
     * Get the number of columns for the model.
     */
    MapDTableModel.prototype.columnCount = function (region) {
        return region === 'body' ? this._fieldNames.length : 1;
    };
    Object.defineProperty(MapDTableModel.prototype, "connection", {
        /**
         * The current connection data for the model.
         */
        get: function () {
            return this._connection;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapDTableModel.prototype, "query", {
        /**
         * The current query for the viewer.
         */
        get: function () {
            return this._query;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Get data from the model.
     */
    MapDTableModel.prototype.data = function (region, row, column) {
        var _this = this;
        if (region === 'row-header') {
            return String(row + 1);
        }
        if (region === 'column-header') {
            return this._fieldNames[column];
        }
        if (region === 'corner-header') {
            return null;
        }
        // If we are streaming data, first check to see if the
        // relevant block is loaded into memory. If it is not,
        // load it. Also load the blocks on each side of the relevant block,
        // and free blocks outside of that window.
        if (this._streaming) {
            var blockIndex = Math.floor(row / BLOCK_SIZE);
            var offset = BLOCK_SIZE * blockIndex;
            var localRow = row - offset;
            // Trash other blocks that we don't need.
            var keep_1 = [0, blockIndex + 1, blockIndex, blockIndex - 1];
            Object.keys(this._dataBlocks).forEach(function (index) {
                var idx = Number(index);
                if (keep_1.indexOf(idx) === -1) {
                    _this._freeBlock(idx);
                }
            });
            // Check if we should fetch the next block.
            if (blockIndex <= this._maxBlock &&
                localRow / BLOCK_SIZE > 0.9 &&
                !this._dataBlocks[blockIndex + 1]) {
                this._fetchBlock(blockIndex + 1);
            }
            // Check if we should fetch the previous block.
            if (blockIndex >= 1 &&
                localRow / BLOCK_SIZE < 0.1 &&
                !this._dataBlocks[blockIndex - 1]) {
                this._fetchBlock(blockIndex - 1);
            }
            // If the current block has not been loaded, then load it and
            // return null. The grid will be notified when it is loaded.
            // If the current block has been loaded, then return the data from it.
            if (!this._dataBlocks[blockIndex]) {
                this._fetchBlock(blockIndex);
                return null;
            }
            else {
                var block = this._dataBlocks[blockIndex];
                var rowData = block[row - offset];
                return rowData[this._fieldNames[column]];
            }
        }
        else {
            // If we are not streaming, then just return the loaded data.
            var rowData = this._dataset[row];
            return rowData[this._fieldNames[column]];
        }
    };
    /**
     * Update the model with new connection data or a new query.
     *
     * @param connection - the connection data to use.
     *
     * @param query - the query to use.
     *
     * @returns a promsise that resolves when the model has been updated,
     *   and the connection and query data have been validated. It throws
     *   an error if the validation fails.
     */
    MapDTableModel.prototype.updateModel = function (connection, query) {
        if (this._query === query && connection && this._connection
            && coreutils_1.JSONExt.deepEqual(connection, this._connection)) {
            return Promise.resolve(void 0);
        }
        this._query = query;
        this._connection = connection;
        return this._updateModel();
    };
    ;
    /**
     * Reset the model. Should be called when either
     * the query or the connection data change.
     */
    MapDTableModel.prototype._updateModel = function () {
        var _this = this;
        // Clear the data of any previous model
        for (var _i = 0, _a = Object.keys(this._dataBlocks); _i < _a.length; _i++) {
            var key = _a[_i];
            delete this._dataBlocks[Number(key)];
        }
        this._dataset = null;
        this._fieldNames = [];
        this._tableLength = -1;
        this._maxBlock = Infinity;
        this._pending.clear();
        this._streaming = false;
        if (this.query) {
            this._streaming = Private.shouldChunkRequests(this._query);
            return Private.getFields(this._connection, this._query).then(function (names) {
                _this._fieldNames = names;
                _this.emitChanged({ type: 'model-reset' });
                if (_this._streaming) {
                    _this._fetchBlock(0);
                }
                else {
                    _this._fetchDataset();
                }
            }).catch(function (err) {
                _this.emitChanged({ type: 'model-reset' });
                throw err;
            });
        }
        else {
            this.emitChanged({ type: 'model-reset' });
            return Promise.resolve(void 0);
        }
    };
    /**
     * Fetch a block with a given index into memory.
     */
    MapDTableModel.prototype._fetchBlock = function (index) {
        var _this = this;
        // If we are already fetching this block, do nothing.
        if (this._pending.has(index)) {
            return;
        }
        this._pending.add(index);
        // Augment the query with the relevant LIMIT and OFFSET.
        var limit = BLOCK_SIZE;
        var offset = index * BLOCK_SIZE;
        var query = this._query + " LIMIT " + limit + " OFFSET " + offset;
        var indices = Object.keys(this._dataBlocks).map(function (key) { return Number(key); });
        var maxIndex = Math.max.apply(Math, indices);
        Private.makeQuery(this._connection, query).then(function (res) {
            _this._pending.delete(index);
            _this._dataBlocks[index] = res;
            if (index <= maxIndex || _this._tableLength !== -1) {
                // In this case, we are not appending, so emit a changed
                // signal.
                _this.emitChanged({
                    type: 'cells-changed',
                    region: 'body',
                    rowIndex: offset,
                    columnIndex: 0,
                    rowSpan: res.length,
                    columnSpan: _this._fieldNames.length,
                });
            }
            else {
                if (res.length < BLOCK_SIZE) {
                    // If the length of the result is less than the block size,
                    // we have found the table length. Set that.
                    _this._tableLength = offset + res.length;
                    _this._maxBlock = index;
                }
                // Emit a rows-inserted signal.
                _this.emitChanged({
                    type: 'rows-inserted',
                    region: 'body',
                    index: offset,
                    span: res.length,
                });
            }
        });
    };
    /**
     * Free references to a block when it is no longer needed.
     */
    MapDTableModel.prototype._freeBlock = function (index) {
        if (!this._dataBlocks[index]) {
            return;
        }
        var offset = index * BLOCK_SIZE;
        var length = this._dataBlocks[index].length;
        delete this._dataBlocks[index];
        this.emitChanged({
            type: 'cells-changed',
            region: 'body',
            rowIndex: offset,
            columnIndex: 0,
            rowSpan: length,
            columnSpan: this._fieldNames.length,
        });
    };
    /**
     * If we are not chunking the data, then just load the whole thing,
     * limited by DEFAULT_LIMIT.
     */
    MapDTableModel.prototype._fetchDataset = function () {
        var _this = this;
        Private.makeQuery(this._connection, this._query, { limit: DEFAULT_LIMIT }).then(function (res) {
            _this._tableLength = res.length;
            // If the dataset already exists, emit a cells-changed signal.
            // Otherwise, emit a 'rows-inserted' signal.
            if (_this._dataset) {
                _this._dataset = res;
                _this.emitChanged({
                    type: 'cells-changed',
                    region: 'body',
                    rowIndex: 0,
                    columnIndex: 0,
                    rowSpan: res.length,
                    columnSpan: _this._fieldNames.length,
                });
            }
            else {
                _this._dataset = res;
                _this.emitChanged({
                    type: 'rows-inserted',
                    region: 'body',
                    index: 0,
                    span: res.length,
                });
            }
        });
    };
    return MapDTableModel;
}(datagrid_1.DataModel));
exports.MapDTableModel = MapDTableModel;
var Private;
(function (Private) {
    /**
     * Whether to chunk requests to the backend. We only do this is if
     * (1) LIMIT/OFFSET has not been defined.
     * (2) ORDER BY has been defined, otherwise we cannot guarantee a consistent
     * ordering across requests.
     */
    function shouldChunkRequests(query) {
        return query.search(/limit/i) === -1 && query.search(/offset/i) === -1
            && query.search(/order by/i) !== -1;
    }
    Private.shouldChunkRequests = shouldChunkRequests;
    /**
     * Validate a query, getting the fields that will be returned by the query.
     */
    function getFields(connection, query) {
        return validateQuery(connection, query).then(function (res) {
            return res.map(function (item) { return item.name; });
        });
    }
    Private.getFields = getFields;
    /**
     * Query the MapD backend.
     */
    function makeQuery(connection, query, options) {
        if (options === void 0) { options = {}; }
        return new Promise(function (resolve, reject) {
            new MapdCon()
                .protocol(connection.protocol)
                .host(connection.host)
                .port(connection.port)
                .dbName(connection.dbName)
                .user(connection.user)
                .password(connection.password)
                .connect(function (error, con) {
                if (error) {
                    reject(error);
                }
                else {
                    con.query(query, options, function (err, result) {
                        if (err) {
                            reject(err);
                        }
                        else {
                            resolve(result);
                        }
                    });
                }
            });
        });
    }
    Private.makeQuery = makeQuery;
    /**
     * Validate a query with the MapD backend.
     */
    function validateQuery(connection, query) {
        return new Promise(function (resolve, reject) {
            new MapdCon()
                .protocol(connection.protocol)
                .host(connection.host)
                .port(connection.port)
                .dbName(connection.dbName)
                .user(connection.user)
                .password(connection.password)
                .connect(function (error, con) {
                if (error) {
                    reject(error);
                }
                else {
                    con.validateQuery(query).then(function (result) {
                        resolve(result);
                    }).catch(function (err) { reject(err); });
                }
            });
        });
    }
})(Private || (Private = {}));
