"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
Object.defineProperty(exports, "__esModule", { value: true });
var widgets_1 = require("@phosphor/widgets");
var widget_1 = require("./widget");
require("../style/index.css");
/**
 * The MIME type for backend-rendered MapD.
 */
exports.MIME_TYPE = 'application/vnd.mapd.vega+json';
/**
 * The MIME type for png data.
 */
exports.IMAGE_MIME = 'image/png';
/**
 * A class for rendering a MapD-generated image.
 */
var RenderedMapD = /** @class */ (function (_super) {
    __extends(RenderedMapD, _super);
    function RenderedMapD() {
        var _this = _super !== null && _super.apply(this, arguments) || this;
        _this._widget = null;
        return _this;
    }
    /**
     * Render MapD image into this widget's node.
     */
    RenderedMapD.prototype.renderModel = function (model) {
        // If we have already rendered a widget, dispose of it.
        if (this._widget) {
            this._widget.parent = null;
            this._widget.dispose();
            this._widget = null;
        }
        // If there is png image data in the mimebundle,
        // we can render that instead of making another
        // request to the backend.
        var imageData = model.data[exports.IMAGE_MIME];
        if (imageData) {
            this.node.appendChild(Private.createImageNode(imageData));
            return Promise.resolve(void 0);
        }
        // Get the data from the mimebundle
        var data = model.data[exports.MIME_TYPE];
        var connection = data.connection, vega = data.vega;
        // Create a new MapDVega
        this._widget = new widget_1.MapDVega(vega, connection);
        this.node.appendChild(this._widget.node);
        return this._widget.renderedImage.then(function (data) {
            // Set the mime data for the png.
            // This allows us to re-use the image if
            // we are loading from disk.
            model.setData({
                data: __assign({ 'image/png': data }, model.data),
                metadata: model.metadata
            });
            return void 0;
        });
    };
    return RenderedMapD;
}(widgets_1.Widget));
exports.RenderedMapD = RenderedMapD;
/**
 * A mime renderer factory for mapd-vega data.
 */
exports.rendererFactory = {
    safe: false,
    mimeTypes: [exports.MIME_TYPE],
    defaultRank: 10,
    createRenderer: function (options) { return new RenderedMapD(); }
};
var extensions = [
    {
        id: 'jupyterlab-mapd:factory',
        rendererFactory: exports.rendererFactory,
        dataType: 'string',
    }
];
exports.default = extensions;
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    /**
     * Create an image node from a base64-encoded image.
     */
    function createImageNode(blob) {
        var img = document.createElement('img');
        var blobUrl = "data:" + exports.IMAGE_MIME + ";base64," + blob;
        img.src = blobUrl;
        return img;
    }
    Private.createImageNode = createImageNode;
})(Private || (Private = {}));
