"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var coreutils_1 = require("@phosphor/coreutils");
var widgets_1 = require("@phosphor/widgets");
var apputils_1 = require("@jupyterlab/apputils");
var coreutils_2 = require("@jupyterlab/coreutils");
var docregistry_1 = require("@jupyterlab/docregistry");
var connection_1 = require("./connection");
var widget_1 = require("./widget");
var MapDViewer = /** @class */ (function (_super) {
    __extends(MapDViewer, _super);
    function MapDViewer(context, connection) {
        var _this = _super.call(this) || this;
        _this._ready = new coreutils_1.PromiseDelegate();
        _this._widget = null;
        _this.context = context;
        _this.connection = connection;
        _this._onTitleChanged();
        context.pathChanged.connect(_this._onTitleChanged, _this);
        context.ready.then(function () {
            if (_this.isDisposed) {
                return;
            }
            _this._render();
            context.model.contentChanged.connect(_this.update, _this);
            _this._ready.resolve(void 0);
        });
        _this.layout = new widgets_1.PanelLayout();
        _this._toolbar = new apputils_1.Toolbar();
        _this._toolbar.addClass('mapd-MapD-toolbar');
        _this._content = new widgets_1.Widget();
        _this._content.addClass('mapd-MapDViewer-content');
        _this.layout.addWidget(_this._toolbar);
        _this.layout.addWidget(_this._content);
        _this._toolbar.addItem('Connect', new apputils_1.ToolbarButton({
            className: 'mapd-MapD-logo',
            onClick: function () {
                connection_1.showConnectionDialog(_this._connection).then(function (connection) {
                    _this._connection = connection;
                });
            },
            tooltip: 'Enter MapD Connection Data'
        }));
        _this._toolbar.addItem('Render', new apputils_1.ToolbarButton({
            className: 'jp-RunIcon',
            onClick: function () {
                _this._render();
            },
            tooltip: 'Render'
        }));
        return _this;
    }
    Object.defineProperty(MapDViewer.prototype, "connection", {
        /**
         * The current connection data for the viewer.
         */
        get: function () {
            return this._connection;
        },
        set: function (value) {
            this._connection = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(MapDViewer.prototype, "ready", {
        /**
         * A promise that resolves when the viewer is ready.
         */
        get: function () {
            return this._ready.promise;
        },
        enumerable: true,
        configurable: true
    });
    /**
     * Handle a change to the title.
     */
    MapDViewer.prototype._onTitleChanged = function () {
        this.title.label = coreutils_2.PathExt.basename(this.context.localPath);
    };
    /**
     * Render MapD into this widget's node.
     */
    MapDViewer.prototype._render = function () {
        var _this = this;
        if (this._widget) {
            this._content.node.removeChild(this._widget.node);
            this._widget.dispose();
            this._widget = null;
        }
        var text = this.context.model.toString();
        // If there is no data or no connection, do nothing
        if (!text || !this._connection) {
            return Promise.resolve(void 0);
        }
        var data = JSON.parse(text.replace(/\n/g, ''));
        this._widget = new widget_1.MapDVega(data, this._connection);
        this._content.node.appendChild(this._widget.node);
        var spinner = new apputils_1.Spinner();
        this._content.node.appendChild(spinner.node);
        return this._widget.renderedImage.then(function () {
            _this._content.node.removeChild(spinner.node);
            spinner.dispose();
            return void 0;
        }).catch(function () {
            _this._content.node.removeChild(spinner.node);
            spinner.dispose();
            return void 0;
        });
    };
    return MapDViewer;
}(widgets_1.Widget));
exports.MapDViewer = MapDViewer;
/**
 * A widget factory for images.
 */
var MapDViewerFactory = /** @class */ (function (_super) {
    __extends(MapDViewerFactory, _super);
    function MapDViewerFactory() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    /**
     * Create a new widget given a context.
     */
    MapDViewerFactory.prototype.createNewWidget = function (context) {
        return new MapDViewer(context, this.defaultConnection);
    };
    Object.defineProperty(MapDViewerFactory.prototype, "defaultConnection", {
        /**
         * The current default connection data for viewers.
         */
        get: function () {
            return this._defaultConnection;
        },
        set: function (value) {
            this._defaultConnection = value;
        },
        enumerable: true,
        configurable: true
    });
    return MapDViewerFactory;
}(docregistry_1.ABCWidgetFactory));
exports.MapDViewerFactory = MapDViewerFactory;
