"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var coreutils_1 = require("@phosphor/coreutils");
var widgets_1 = require("@phosphor/widgets");
require("mapd-connector/dist/browser-connector.js");
/**
 * The MIME type for png data.
 */
var IMAGE_MIME = 'image/png';
/**
 * A class for rendering a MapD-generated image.
 */
var MapDVega = /** @class */ (function (_super) {
    __extends(MapDVega, _super);
    /**
     * Construct a new MapD widget.
     */
    function MapDVega(vega, connection) {
        var _this = _super.call(this) || this;
        _this._rendered = new coreutils_1.PromiseDelegate();
        _this.addClass('mapd-MapDVega');
        _this._img = document.createElement('img');
        _this._error = document.createElement('pre');
        _this._error.className = 'mapd-ErrorMessage';
        _this.node.appendChild(_this._img);
        _this.node.appendChild(_this._error);
        _this._connection = connection;
        _this._vega = vega;
        _this._renderData();
        return _this;
    }
    Object.defineProperty(MapDVega.prototype, "renderedImage", {
        get: function () {
            return this._rendered.promise;
        },
        enumerable: true,
        configurable: true
    });
    MapDVega.prototype._renderData = function () {
        var _this = this;
        var connection = this._connection;
        var vega = this._vega;
        new MapdCon()
            .protocol(connection.protocol)
            .host(connection.host)
            .port(connection.port)
            .dbName(connection.dbName)
            .user(connection.user)
            .password(connection.password)
            .connect(function (error, con) {
            if (error) {
                // If there was an error, clear any image data,
                // and set the text content of the error node.
                _this._setImageData('');
                _this._error.textContent = error;
                _this._rendered.reject(error);
                return;
            }
            con.renderVega(Private.id++, JSON.stringify(vega), {}, function (error, result) {
                if (error) {
                    // If there was an error, clear any image data,
                    // and set the text content of the error node.
                    _this._setImageData('');
                    _this._error.textContent = error.message;
                    _this._rendered.reject(error.message);
                }
                else {
                    // Set the image data.
                    _this._setImageData(result.image);
                    // Clear any error message.
                    _this._error.textContent = '';
                    _this._rendered.resolve(result.image);
                }
            });
        });
    };
    /**
     * Set the image data to a base64 encoded string.
     */
    MapDVega.prototype._setImageData = function (blob) {
        var blobUrl = "data:" + IMAGE_MIME + ";base64," + blob;
        this._img.src = blobUrl;
    };
    return MapDVega;
}(widgets_1.Widget));
exports.MapDVega = MapDVega;
/**
 * A namespace for private data.
 */
var Private;
(function (Private) {
    Private.id = 0;
})(Private || (Private = {}));
